(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DecoderError extends Error {
        constructor() {
            super(...arguments);
            this.name = 'DecoderError';
        }
    }
    exports.DecoderError = DecoderError;
    class OneOfDecoderError extends DecoderError {
        constructor(errors) {
            super(`all decoders failed\n\t${errors.map(error => `${error}`).join('\n\t')}`);
            this.name = 'OneOfDecoderError';
        }
    }
    exports.OneOfDecoderError = OneOfDecoderError;
    class AtDecoderError extends DecoderError {
        constructor(path, err) {
            super(`at path: "${path}"\n\t${err}`);
            this.path = path;
            this.name = 'AtDecoderError';
        }
    }
    exports.AtDecoderError = AtDecoderError;
    class ArrayDecoderError extends DecoderError {
        constructor(index, err) {
            super(`at index: "${index}"\n\t${err}`);
            this.index = index;
            this.name = 'ArrayDecoderError';
        }
    }
    exports.ArrayDecoderError = ArrayDecoderError;
    class PropsDecoderError extends DecoderError {
        constructor(key, err) {
            super(`at key: "${key}"\n\t${err}`);
            this.key = key;
            this.name = 'PropsDecoderError';
        }
    }
    exports.PropsDecoderError = PropsDecoderError;
    class ScalarDecoderError extends DecoderError {
        constructor(expectedType, value) {
            super(`expected type: "${expectedType}" but got type: "${typeof value}" for value: "${value}"`);
            this.expectedType = expectedType;
            this.value = value;
            this.name = 'ScalarDecoderError';
        }
    }
    exports.ScalarDecoderError = ScalarDecoderError;
    class StrictDecoderError extends DecoderError {
        constructor(expectedValue, value) {
            super(`expected value: "${expectedValue}" of type: "${typeof expectedValue}" but got value: "${value}" of type: "${typeof value}"`);
            this.expectedValue = expectedValue;
            this.value = value;
            this.name = 'StrictDecoderError';
        }
    }
    exports.StrictDecoderError = StrictDecoderError;
});
//# sourceMappingURL=errors.js.map