var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./index"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const decod = __importStar(require("./index"));
    describe('Decod', () => {
        it('Should decode strings', () => {
            expect(() => decod.string('Hello')).not.toThrowError();
            expect(() => decod.string(12)).toThrowError(`expected type: "string" but got type: "number" for value: "12"`);
            expect(decod.string('Hello')).toEqual('Hello');
        });
        it('Should decode strict values', () => {
            expect(() => decod.is('Hello')('Hello')).not.toThrowError();
            expect(() => decod.is(12)('12')).toThrowError(`expected value: "12" of type: "number" but got value: "12" of type: "string"`);
            expect(decod.is('Hello')('Hello')).toEqual('Hello');
        });
        it('Should decode numbers', () => {
            expect(() => decod.number(12)).not.toThrowError();
            expect(() => decod.number('12')).toThrowError(`expected type: "number" but got type: "string" for value: "12"`);
            expect(decod.number(12)).toEqual(12);
        });
        it('Should decode booleans', () => {
            expect(() => decod.boolean(false)).not.toThrowError();
            expect(() => decod.boolean('32')).toThrowError(`expected type: "boolean" but got type: "string" for value: "32"`);
            expect(decod.boolean(false)).toEqual(false);
        });
        it('Should decode dates', () => {
            expect(() => decod.date(new Date())).not.toThrowError();
            expect(() => decod.date(32)).toThrowError(`expected type: "Date" but got type: "number" for value: "32"`);
            const date = new Date();
            expect(decod.date(date)).toEqual(date);
        });
        it('Should decode nulls', () => {
            expect(() => decod.null_(null)).not.toThrowError();
            expect(() => decod.null_('32')).toThrowError(`expected type: "null" but got type: "string" for value: "32"`);
            expect(decod.null_(null)).toEqual(null);
        });
        it('Should follow the path', () => {
            expect(decod.at(2, decod.string)(['foo', 'bar', 'baz'])).toEqual('baz');
            expect(decod.at('foo', decod.string)({ foo: 'bar', quux: 'baz' })).toEqual('bar');
            expect(decod.at(['foo', 1, 'bar'], decod.number)({
                foo: [{ bar: 0 }, { bar: 42 }],
            })).toEqual(42);
        });
        it('Should use the props helper', () => {
            expect(() => decod.props({})('hello')).not.toThrowError();
            expect(() => decod.props({ val: decod.number })('hello')).toThrowError(`expected type: "number" but got type: "string" for value: "hello"`);
            expect(decod.props({
                one: decod.at('one', decod.string),
                two: decod.at(['two', 'nested'], decod.number),
                three: decod.at(['three', 1], decod.string),
            })({ one: 'one', two: { nested: 2 }, three: ['n/a', 'three'] })).toEqual({ one: 'one', two: 2, three: 'three' });
        });
        it('Should decode nullables', () => {
            expect(() => decod.nullable(decod.string)(null)).not.toThrowError();
            expect(() => decod.nullable(decod.string)('null')).not.toThrowError();
            expect(() => decod.nullable(decod.string)(32)).toThrowError(`all decoders failed\n\tScalarDecoderError: expected type: "string" but got type: "number" for value: "32"\n\tScalarDecoderError: expected type: "null" but got type: "number" for value: "32"`);
            expect(decod.nullable(decod.string)('null')).toBe('null');
            expect(decod.nullable(decod.string)(null)).toBe(null);
        });
        it('Should decode optionals', () => {
            expect(() => decod.optional(decod.string)(null)).not.toThrowError();
            expect(() => decod.optional(decod.string)('null')).not.toThrowError();
            expect(() => decod.optional(decod.string)(32)).toThrowError(`all decoders failed\n\tScalarDecoderError: expected type: "string" but got type: "number" for value: "32"\n\tScalarDecoderError: expected type: "null" but got type: "number" for value: "32"\n\tScalarDecoderError: expected type: "undefined" but got type: "number" for value: "32"`);
            expect(decod.optional(decod.string)('null')).toBe('null');
            expect(decod.optional(decod.string)(null)).toBe(null);
            expect(decod.optional(decod.string)(undefined)).toBe(undefined);
        });
        it('Should decode undefined', () => {
            expect(() => decod.undefined_(undefined)).not.toThrowError();
            expect(() => decod.undefined_('32')).toThrowError(`expected type: "undefined" but got type: "string" for value: "32"`);
            expect(decod.undefined_(undefined)).toEqual(undefined);
        });
        it('Should try to decode', () => {
            expect(() => decod.try_(decod.string)('32')).not.toThrowError();
            expect(() => decod.try_(decod.string)(32)).not.toThrowError();
            expect(decod.try_(decod.string)('32')).toEqual('32');
        });
        it('Should decode assoc', () => {
            const valueDecoder = decod.oneOf(decod.string, decod.number);
            const decoder = decod.assoc(decod.string, valueDecoder);
            expect(() => decoder({ a: 'b' })).not.toThrowError();
            expect(decoder({ a: 'one', b: 1 })).toEqual([
                {
                    key: 'a',
                    value: 'one',
                },
                {
                    key: 'b',
                    value: 1,
                },
            ]);
        });
        it('Should decode objects', () => {
            expect(() => decod.at(['a', 'b', 1, 2], decod.string)({ a: { b: { 1: { 2: '' } } } })).not.toThrowError();
            expect(() => decod.at(['a'], decod.string)('')).toThrowError(`at path: "a"\n\tScalarDecoderError: expected type: "string" but got type: "undefined" for value: "undefined"`);
            expect(decod.at(['a'], decod.string)({ a: 'Hello' })).toEqual('Hello');
        });
        it('Should decode arrays', () => {
            expect(() => decod.array(decod.string)(['toto', 'titi', 'tata'])).not.toThrowError();
            expect(() => decod.array(decod.string)([32, 32, 12])).toThrowError(`at index: "0"\n\tScalarDecoderError: expected type: "string" but got type: "number" for value: "32"`);
            expect(() => decod.array(decod.string)('Hello')).toThrowError(`expected type: "array" but got type: "string" for value: "Hello"`);
            expect(decod.array(decod.string)(['toto', 'titi', 'tata'])).toEqual([
                'toto',
                'titi',
                'tata',
            ]);
        });
        it('Should decode one of things', () => {
            expect(() => decod.oneOf(decod.string, decod.boolean)(true)).not.toThrowError();
            expect(() => decod.oneOf(decod.string, decod.boolean)(32)).toThrowError(`all decoders failed\n\tScalarDecoderError: expected type: "string" but got type: "number" for value: "32"\n\tScalarDecoderError: expected type: "boolean" but got type: "number" for value: "32"`);
            expect(decod.oneOf(decod.string, decod.boolean)(true)).toEqual(true);
        });
    });
});
//# sourceMappingURL=index.test.js.map